from deemon.app import settings, db


class Deemon:
    def __init__(self):
        self.settings = settings.Settings()
        self.config = self.settings.config
        self.db = db.DBHelper(self.settings.db_path)



from deemon.app import settings, db
from deemon.app import Deemon
from operator import itemgetter
import logging
import time
import sys

logger = logging.getLogger(__name__)


class ShowStats(Deemon):

    def __init__(self):
        super().__init__()

    def artists(self):
        monitored_artists = self.db.get_all_monitored_artists()
        if len(monitored_artists) == 0:
            logger.info("No artists are being monitored")
            sys.exit(0)

        artist_data = [artist[1] for artist in monitored_artists]

        if len(artist_data) > 10:
            artist_data = self.truncate_long_artists(artist_data)

            if len(artist_data) % 2 != 0:
                artist_data.append(" ")

            for a, b in zip(artist_data[0::2], artist_data[1::2]):
                print('{:<30}{:<}'.format(a, b))
        else:
            for artist in artist_data:
                print(artist)

    @staticmethod
    def truncate_long_artists(all_artists):
        for idx, artist in enumerate(all_artists):
            if len(artist) > 25:
                all_artists[idx] = artist[:22] + "..."
        return all_artists

    def releases(self, days):
        seconds_per_day = 86400
        days_in_seconds = (days * seconds_per_day)
        now = int(time.time())
        back_date = (now - days_in_seconds)
        releases = self.db.show_new_releases(back_date, now)
        release_list = [x for x in releases]
        if len(release_list) > 0:
            logger.info(f"New releases found within last {days} day(s):")
            print("")
            release_list.sort(key=itemgetter(4), reverse=True)
            for release in release_list:
                print('+ [%-10s] %s - %s' % (release[4], release[1], release[3]))
        else:
            logger.info(f"No releases found in that timeframe")

    def stats(self):
        pass



from sqlite3 import OperationalError
from deemon.app import db, settings
import logging
import deezer

logger = logging.getLogger(__name__)


class Monitor:

    def __init__(self, artist=None, artist_id=None):
        self.settings = settings.Settings()
        self.config = self.settings.config
        self.artist = artist
        self.artist_id = artist_id
        self.db = db.DBHelper(self.settings.db_path)
        self.dz = deezer.Deezer()

    def get_artist_info(self):
        if self.artist_id:
            try:
                artist = self.dz.api.get_artist(self.artist_id)
                self.artist = artist["name"]
                return True
            except deezer.api.DataException:
                logger.error(f"Artist ID '{self.artist_id}' not found.")

        if self.artist:
            try:
                artist = self.dz.api.search_artist(self.artist, limit=1)["data"][0]
                self.artist = artist["name"]
                self.artist_id = artist["id"]
                return True
            except IndexError:
                logger.error(f"Artist '{self.artist}' not found")

    def start_monitoring(self):
        artist_info = self.get_artist_info()
        values = {'artist_name': self.artist}
        sql = "SELECT * FROM monitor WHERE artist_name = :artist_name"
        already_monitored = self.db.query(sql, values).fetchone()
        if already_monitored:
            logger.debug(f"Artist: {self.artist} is already monitored, skipping...")
            return
        if artist_info:
            sql = ("INSERT OR REPLACE INTO monitor (artist_id, artist_name, bitrate, record_type, alerts) "
                   "VALUES (:artist_id, :artist_name, :bitrate, :record_type, :alerts)")
            values = {
                'artist_id': self.artist_id,
                'artist_name': self.artist,
                'bitrate': self.config["bitrate"],
                'record_type': self.config["record_type"],
                'alerts': self.config["alerts"]
            }

            try:
                self.db.query(sql, values)
            except OperationalError as e:
                logger.error(e)

            logger.info(f"Now monitoring {self.artist}")

            self.db.commit()
            return 1
        else:
            return 2

    def stop_monitoring(self):
        self.get_artist_info()
        if self.artist_id:
            values = {'artist_id': self.artist_id}
            sql_releases = "DELETE FROM 'releases' WHERE artist_id = :artist_id"
            sql_monitor = "DELETE FROM 'monitor' WHERE artist_id = :artist_id"
        else:
            values = {'artist': self.artist}
            sql_releases = "DELETE FROM 'releases' WHERE artist_name = :artist COLLATE NOCASE"
            sql_monitor = "DELETE FROM 'monitor' WHERE artist_name = :artist COLLATE NOCASE"

        result = self.db.query(sql_monitor, values)
        if result.rowcount > 0:
            logger.info("No longer monitoring " + self.artist)
            logger.info("Cleaning up release table...")
            self.db.query(sql_releases, values)
        else:
            logger.error(f"Artist '{self.artist}' not found")

        self.db.commit()



from deemon.app import Deemon, download, monitor
from deemon.app.refresh import Refresh
from pathlib import Path
import tqdm
import logging
import sys

logger = logging.getLogger(__name__)


class BatchJobs(Deemon):

    def import_artists(self, path):
        import_artists = path
        # TODO check db for existing artist
        if import_artists:
            if Path(import_artists).is_file():
                with open(import_artists, encoding="utf8") as f:
                    import_list = f.read().splitlines()
                    num_to_import = len(import_list)
                    logger.debug(f"Detected {num_to_import} artist(s) to import")
            elif Path(import_artists).is_dir():
                import_list = [x for x in Path(import_artists).iterdir() if x.is_dir()]
                num_to_import = len(import_list)
                logger.debug(f"Detected {num_to_import} artist(s) to import")
            else:
                logger.error(f"File or directory not found: {import_artists}")
                sys.exit(1)

            progress = tqdm.tqdm(import_list, bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt} ({elapsed}{postfix})')

            import_count = 0
            import_error = 0
            for artist in progress:
                progress.set_description("Importing")
                ma = monitor.Monitor()
                ma.artist = artist
                imported = ma.start_monitoring()
                if imported == 1:
                    import_count += 1
                elif imported == 2:
                    import_error += 1
            if import_count > 0:
                logger.info(f"** Successful: {import_count} | Errors: {import_error}")
            else:
                logger.info(f"** No new artists to import | Errors: {import_error}")
            self.db.commit()

            print()
            refresh = Refresh()
            refresh.refresh()



from deemon.app import utils
from pathlib import Path
import logging
import json

logger = logging.getLogger(__name__)

DEFAULT_SETTINGS = {
    "plex_baseurl": "",
    "plex_token": "",
    "plex_library": "",
    "download_path": "",
    "deemix_path": "",
    "release_by_date": 1,
    "release_max_days": 90,
    "bitrate": "320",
    "alerts": 0,
    "record_type": "all",
    "smtp_server": "",
    "smtp_port": 465,
    "smtp_user": "",
    "smtp_pass": "",
    "smtp_sender": "",
    "smtp_recipient": ""
}


class Settings:

    def __init__(self, custom_path=None):
        self.config_file = 'config.json'
        self.db_file = 'deemon.db'
        self.config_path = utils.get_appdata_dir()
        self.db_path = Path(self.config_path / self.db_file)

        if not Path(self.config_path / self.config_file).exists():
            self.create_default_config()

        self.config = self.load_config()
        self.verify_config()

    def create_default_config(self):
        with open(self.config_path / 'config.json', 'w') as f:
            json.dump(DEFAULT_SETTINGS, f, indent=4)

    def load_config(self):
        with open(Path(self.config_path / self.config_file)) as f:
            return json.load(f)

    def verify_config(self):
        for group in DEFAULT_SETTINGS:
            if group not in self.config:
                self.config[group] = DEFAULT_SETTINGS[group]



from pathlib import Path

import plexapi.exceptions
from plexapi.server import PlexServer
from deemon.app import dmi, notify
from deemon.app import Deemon
import logging
import deezer
import sys

logger = logging.getLogger(__name__)


class QueueItem:

    def __init__(self, artist: dict, album: dict):
        self.artist_name = artist["name"]
        self.bitrate = artist["bitrate"]
        self.album_id = album["id"]
        self.album_title = album["title"]
        self.url = album["link"]


class Download(Deemon):

    def __init__(self, login=True):
        super().__init__()
        self.dz = deezer.Deezer()
        self.di = dmi.DeemixInterface(self.config["download_path"], self.config["deemix_path"])
        self.deemix_logger = logging.getLogger("deemix")
        self.queue_list = []
        self.bitrate = self.config["bitrate"]
        self.record_type = self.config["record_type"]

        if login:
            if not self.di.login():
                sys.exit(1)

    def get_plex_server(self):
        baseurl = self.config["plex_baseurl"]
        token = self.config["plex_token"]
        if (baseurl != "") and (token != ""):
            try:
                logger.info("----------------------------")
                logger.info("Plex settings found! Trying to connect...")
                plex_server = PlexServer(baseurl, token, timeout=10)
                return plex_server
            except Exception:
                logger.error("Error: Unable to reach Plex server, please refresh manually.")
                return False

    def download_queue(self, queue):
        if queue:
            plex = self.get_plex_server()
            num_queued = len(queue)
            logger.info("----------------------------")
            logger.info("Sending " + str(num_queued) + " release(s) to deemix for download:")

            for q in queue:
                logger.info(f"+ {q.artist_name} - {q.album_title}... ")
                self.di.download_url([q.url], q.bitrate)

            print("")
            logger.info("Downloads complete!")
            if plex:
                try:
                    plex.library.section(self.config["plex_library"]).update()
                    logger.debug("Plex library refreshed successfully")
                except plexapi.exceptions.BadRequest as e:
                    logger.error("Error occurred while refreshing your library. See logs for additional info.")
                    logger.debug(f"Error during Plex refresh: {e}")

    def add_to_queue(self, artist, album):
        for _album in album['data']:
            if (self.record_type == _album["record_type"]) or (self.record_type == "all"):
                logger.debug(f"QUEUE: Adding '{_album['title']}' to queue...")
                artist["bitrate"] = self.bitrate
                self.queue_list.append(QueueItem(artist, _album))

    def download(self, opt: dict):
        logger.debug("download called with options: " + str(opt))
        artist = {}

        if opt["bitrate"]:
            self.bitrate = opt["bitrate"]
        if opt["record_type"]:
            self.record_type = opt["record_type"]

        if opt["url"]:
            logger.info("Sending URL to deemix for processing:")
            self.deemix_logger.setLevel(logging.INFO)
            self.di.download_url([opt["url"]], opt["bitrate"])

        if opt["file"]:
            logger.info(f"Reading from file {opt['file']}")
            if Path(opt['file']).exists():
                with open(opt['file'], 'r') as f:
                    make_csv = f.read().replace('\n', ',')
                    csv_to_list = make_csv.split(',')
                    artist_list = list(filter(None, csv_to_list))
                    for name in artist_list:
                        try:
                            artist = self.dz.api.search_artist(name, limit=1)['data'][0]
                            album = self.dz.api.get_artist_albums(artist["id"])
                            self.add_to_queue(artist, album)
                        except IndexError:
                            logger.warning(f"Artist '{name}' not found")

        if opt["artist_id"] or opt["artist"]:
            if opt["artist_id"]:
                artist = self.dz.api.get_artist(opt["artist_id"])
            elif opt["artist"]:
                artist = self.dz.api.search_artist(opt["artist"], limit=1)['data'][0]
                opt["artist_id"] = artist["id"]

            album = self.dz.api.get_artist_albums(opt["artist_id"])
            self.add_to_queue(artist, album)

        if opt["album_id"]:
            album = {'data': [self.dz.api.get_album(opt["album_id"])]}
            artist["name"] = album["data"][0]["artist"]["name"]
            self.add_to_queue(artist, album)

        self.download_queue(self.queue_list)
        self.db.commit()



from deemix.app import deemix
from pathlib import Path
import logging

logger = logging.getLogger(__name__)


class DeemixInterface(deemix):
    def __init__(self, download_path, config_dir=None):
        logger.debug("Initializing deemix library")
        super().__init__(config_dir, overwriteDownloadFolder=download_path)

    def download_url(self, urls: list, bitrate: int):
        for url in urls:
            if ';' in url:
                for link in url.split(";"):
                    self.qm.addToQueue(self.dz, link, self.set.settings, bitrate)
            else:
                self.qm.addToQueue(self.dz, url, self.set.settings, bitrate)

    def login(self):
        logger.info("Verifying ARL is valid, please wait...")
        config_dir = Path(self.set.configFolder)
        if Path(config_dir).is_dir():
            if Path(config_dir / '.arl').is_file():
                with open(config_dir / '.arl', 'r') as f:
                    arl = f.readline().rstrip("\n")
                    logger.debug(f"ARL found: {arl}")
                if not self.dz.login_via_arl(arl):
                    logger.error(f"ARL is expired or invalid")
                    return False
            else:
                logger.error(f"ARL not found in {config_dir}")
                return False
        else:
            logger.error(f"ARL directory {config_dir} was not found")
            return False
        return True





from packaging.version import parse as parse_version
from deemon import __version__
from datetime import datetime
from pathlib import Path
import requests
import time
import sys
import os


def get_appdata_dir():
    """
    Get appdata directory where configuration and data is stored
    """
    home_dir = Path.home()

    if os.getenv("XDG_CONFIG_HOME"):
        appdata_dir = Path(os.getenv("XDG_CONFIG_HOME")) / 'deemon'
    elif os.getenv("APPDATA"):
        appdata_dir = Path(os.getenv("APPDATA")) / "deemon"
    elif sys.platform.startswith('darwin'):
        appdata_dir = home_dir / 'Library' / 'Application Support' / 'deemon'
    else:
        appdata_dir = home_dir / '.config' / 'deemon'

    return appdata_dir


def init_appdata_dir(appdata):
    Path(appdata / 'logs').mkdir(parents=True, exist_ok=True)
    Path(appdata / 'backups').mkdir(exist_ok=True)


def get_log_file():
    """
    Get path to log file
    """
    return Path(get_appdata_dir() / 'logs' / 'deemon.log')


def get_todays_date():
    now_ts = int(time.time())
    today_date = datetime.utcfromtimestamp(now_ts).strftime('%Y-%m-%d')
    return today_date


def get_max_release_date(days):
    day_in_secs = 86400
    input_days_in_secs = days * day_in_secs
    max_date_ts = int(time.time()) - input_days_in_secs
    max_date = datetime.utcfromtimestamp(max_date_ts).strftime('%Y-%m-%d')
    return max_date

def check_version():
    latest_ver = "https://api.github.com/repos/digitalec/deemon/releases/latest"
    try:
        response = requests.get(latest_ver)
    except requests.exceptions.ConnectionError:
        return
    local_version = __version__
    remote_version = response.json()["name"]
    if parse_version(remote_version) > parse_version(local_version):
        return remote_version



import smtplib
import platform
import ssl
import logging
from datetime import datetime
from email.utils import formataddr
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage
from email.mime.text import MIMEText
from email.message import EmailMessage

import pkg_resources

from deemon.app import Deemon, utils
from deemon import __version__

logger = logging.getLogger(__name__)


class Notify(Deemon):

    def __init__(self, new_releases: list = None):
        super().__init__()
        logger.debug("notify initialized")
        logger.debug(f"releases to notify on: {new_releases}")
        self.server = self.config["smtp_server"]
        self.port = self.config["smtp_port"]
        self.user = self.config["smtp_user"]
        self.passwd = self.config["smtp_pass"]
        self.sender = self.config["smtp_sender"]
        self.recipient = self.config["smtp_recipient"]
        self.update = utils.check_version()
        self.subject = "New releases detected!"
        self.releases = new_releases

    def send(self, body=None, test=False):
        """
        Send email notification message
        """
        if not all([self.server, self.port, self.user, self.passwd, self.sender, self.recipient]):
            if test:
                logger.info("Unable to send test notification, email is not configured")
            logger.debug("Email not configured, no notifications will be sent")
            return False

        if not body:
            body = self.build_message()

        context = ssl.create_default_context()

        logger.debug("Sending new release notification email")
        logger.debug(f"Using server: {self.server}:{self.port}")

        with smtplib.SMTP_SSL(self.server, self.port, context=context) as server:
            try:
                server.login(self.user, self.passwd)
                server.sendmail(self.sender, self.recipient, body.as_string())
                logger.debug("Email notification has been sent")
            except Exception as e:
                logger.error("Error while sending mail: " + str(e))

    def get_cover_images(self):
        pass

    def build_message(self):
        """
        Builds message by combining plaintext and HTML messages for sending
        """
        msg = MIMEMultipart('mixed')
        msg['To'] = self.recipient
        msg['From'] = formataddr(('deemon', self.sender))
        msg['Subject'] = self.subject
        # part1 = MIMEText(self.plaintext(), 'plain')
        part2 = MIMEText(self.html(), 'html')
        # msg.attach(part1)
        msg.attach(part2)

        logo = pkg_resources.resource_filename('deemon', 'assets/images/logo.png')
        github =  pkg_resources.resource_filename('deemon', 'assets/images/github.png')
        reddit =  pkg_resources.resource_filename('deemon', 'assets/images/reddit.png')
        discord =  pkg_resources.resource_filename('deemon', 'assets/images/discord.png')

        with open(logo, 'rb') as f:
            image = MIMEImage(f.read())
            image.add_header('Content-ID', 'logo')
            msg.attach(image)

        with open(github, 'rb') as f:
            image = MIMEImage(f.read())
            image.add_header('Content-ID', 'github')
            msg.attach(image)

        with open(reddit, 'rb') as f:
            image = MIMEImage(f.read())
            image.add_header('Content-ID', 'reddit')
            msg.attach(image)

        with open(discord, 'rb') as f:
            image = MIMEImage(f.read())
            image.add_header('Content-ID', 'discord')
            msg.attach(image)

        return msg

    def test(self):
        """
        Verify SMTP settings by sending test email
        """
        self.subject = "deemon Test Notification"
        message = "Congrats! You'll now receive new release notifications."
        msg = EmailMessage()
        msg['To'] = self.recipient
        msg['From'] = formataddr(('deemon', self.sender))
        msg['Subject'] = self.subject
        msg.set_content(message)
        self.send(msg, test=True)

    def plaintext(self) -> str:
        """
        Plaintext version of email to send
        """
        message = "The following new releases were detected:\n\n"
        for release in self.releases:
            release_date_ts = datetime.strptime(release["release_date"], "%Y-%m-%d")
            release_date_str = datetime.strftime(release_date_ts, "%A, %B %-d")
            message += f"\n{release_date_str}\n"
            for album in release["releases"]:
                message += f"+ {album['artist']} - {album['album']}\n"
        return message

    def html(self):

        app_version = f"deemon {__version__}"
        py_version = f"python {platform.python_version()}"
        sys_version = f"{platform.system()} {platform.release()}"

        new_release_list_spacer = f"""
            </ul>
            <p style="font-size: 14px; line-height: 140%;">&nbsp;</p>
        """

        all_new_releases = ""

        for release in self.releases:

            if all_new_releases != "":
                all_new_releases += new_release_list_spacer

            release_date_ts = datetime.strptime(release["release_date"], "%Y-%m-%d")
            release_date_str = datetime.strftime(release_date_ts, "%A, %B %-d")

            new_release_list_header = f"""
                <p style="font-size: 14px; line-height: 140%;">
                    <strong>
                        <span style="font-size: 16px; line-height: 22.4px;">{release_date_str}</span>
                    </strong>
                </p>
                <ul>
            """

            new_release_list_item = ""

            for album in release["releases"]:
                new_release_list_item += f"""
                        <li style="font-size: 14px; line-height: 19.6px;">
                            <span style="font-size: 16px; line-height: 22.4px;">
                                {album['artist']} - {album['album']}
                            </span>
                        </li>
                """

            all_new_releases += new_release_list_header + new_release_list_item

        index =  pkg_resources.resource_filename('deemon', 'assets/index.html')
        with open(index, 'r') as f:
            html_output = f.read()

            if self.update:
                html_output = html_output.replace("{UPDATE_MESSAGE}", "A new update is available!")
            else:
                html_output = html_output.replace("{UPDATE_MESSAGE}", "")
                html_output = html_output.replace("{VIEW_UPDATE_MESSAGE}", "display:none;")

            html_output = html_output.replace("{NEW_RELEASE_LIST}", all_new_releases)
            html_output = html_output.replace("{DEEMON_VER}", app_version)
            html_output = html_output.replace("{PY_VER}", py_version)
            html_output = html_output.replace("{SYS_VER}", sys_version)

        return html_output



from deemon.app import Deemon, utils, download, notify
import tqdm
import logging
import deezer

logger = logging.getLogger(__name__)


class Refresh(Deemon):

    def __init__(self, artist_id: list = None, skip_download: bool = False):
        super().__init__()

        if artist_id is None:
            self.artist_id = []

        self.skip_download = skip_download
        self.dz = deezer.Deezer()
        self.new_release_count = 0
        self.monitored_artists = []
        self.queue_list = []
        self.new_releases = []
        self.release_date = ""

    def is_future_release(self):
        today = utils.get_todays_date()
        if self.release_date > today:
            return 1
        else:
            return 0

    def get_monitored_artists(self):
        if len(self.artist_id) == 0:
            self.monitored_artists = self.db.get_all_monitored_artists()
        else:
            for artist in self.artist_id:
                self.monitored_artists.append(self.db.get_all_specified_artist(artist))

    def construct_new_release_list(self, release_date, artist, album, cover):
        for days in self.new_releases:
            for key in days:
                if key == "release_date":
                    if release_date in days[key]:
                        days["releases"].append({'artist': artist, 'album': album, 'cover': cover})
                        return

        self.new_releases.append({'release_date': release_date, 'releases': [{'artist': artist, 'album': album}]})

    def refresh(self):
        logger.debug(f"Refreshing artists")
        self.get_monitored_artists()

        if len(self.monitored_artists) == 0:
            logger.info("At least one artist needs to be monitored before you can refresh!")
            return

        progress = tqdm.tqdm(self.monitored_artists, bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt} '
                                                                '({elapsed}{postfix})')

        for artist in progress:
            progress.set_description("Refreshing")
            self.new_release_count = 0
            new_artist = False
            artist = {"id": artist[0], "name": artist[1], "bitrate": self.config["bitrate"], "record_type": artist[3]}
            artist_exists = self.db.get_artist_by_id(artist_id=artist["id"])
            albums = self.dz.api.get_artist_albums(artist["id"])

            if not artist_exists:
                new_artist = True
                logger.debug(f"New artist detected: {artist['name']}, future releases will be downloaded")

            if new_artist:
                if len(albums["data"]) == 0:
                    logger.warning(f"WARNING: Artist '{artist['name']}' setup for monitoring but no releases were found.")

            for album in albums["data"]:

                already_exists = self.db.get_album_by_id(album_id=album["id"])
                todays_date = utils.get_todays_date()

                if already_exists:
                    release = [x for x in already_exists]
                    _album = {
                        'artist_id': release[0],
                        'artist_name': release[1],
                        'album_id': release[2],
                        'album_name': release[3],
                        'release_date': release[4],
                        'future_release': release[6]
                    }

                    if _album["future_release"] and (_album["release_date"] <= todays_date):
                        logger.info(f"Artist: {_album['artist_name']} ** Pre-release has now been released "
                                    f"and will be downloaded **")
                        self.db.reset_future(_album['album_id'])
                    else:
                        continue
                else:
                    self.db.add_new_release(
                        artist["id"],
                        artist["name"],
                        album["id"],
                        album["title"],
                        album["release_date"],
                        future_release=self.is_future_release()
                    )

                if self.skip_download or new_artist:
                    continue

                self.new_release_count += 1

                if (self.config["record_type"] == album["record_type"]) or (self.config["record_type"] == "all"):
                    if self.config["release_by_date"]:
                        max_release_date = utils.get_max_release_date(self.config["release_max_days"])
                        if album['release_date'] < max_release_date:
                            logger.debug(f"Release '{artist['name']} - {album['title']}' skipped, too old...")
                            continue
                    logger.debug(f"queue: added {artist['name']} - {album['title']} to the queue")
                    self.queue_list.append(download.QueueItem(artist, album))
                    self.construct_new_release_list(album['release_date'], artist['name'],
                                                    album['title'], album['cover_medium'])

            if self.new_release_count > 0:
                logger.info(f"{artist['name']}: {self.new_release_count} new release(s)")

        logger.debug("Refresh complete")
        if self.queue_list:
            print("")
            dl = download.Download()
            dl.download_queue(self.queue_list)
        self.db.commit()

        if len(self.new_releases) > 0 and self.config["alerts"] == 1:
            notification = notify.Notify(self.new_releases)
            notification.send()



from deemon.app import settings, monitor, download, notify
from deemon.app.logger import setup_logger
from deemon.app.batch import BatchJobs
from deemon.app.refresh import Refresh
from deemon.app.show import ShowStats
from deemon import __version__
from datetime import datetime
from deemon.app import utils
from pathlib import Path
import tarfile
import logging
import click
import sys

logger = logging.getLogger(__name__)

appdata = utils.get_appdata_dir()
utils.init_appdata_dir(appdata)
settings = settings.Settings()
settings.load_config()
config = settings.config


@click.group()
@click.option('-v', '--verbose', is_flag=True, help='Enable verbose output')
@click.version_option(__version__, '--version', message='deemon %(version)s')
def run(verbose):
    """Monitoring and alerting tool for new music releases using the Deezer API.

    deemon is a free and open source tool. To report issues or to contribute,
    please visit https://github.com/digitalec/deemon
    """
    setup_logger(log_level='DEBUG' if verbose else 'INFO', log_file=utils.get_log_file())
    new_version = utils.check_version()
    if new_version:
        print("*" * 50)
        logger.info(f"* New version is available: v{__version__} -> v{new_version}")
        logger.info("* To upgrade, run `pip install --upgrade deemon`")
        print("*" * 50)


@run.command(name='test')
def test():
    """Test email server settings by sending a test notification"""
    notification = notify.Notify()
    notification.test()


@run.command(name='download')
@click.option('-a', '--artist', metavar='NAME', type=str, help='Download all by artist name')
@click.option('-i', '--artist-id', metavar='ID', type=int, help='Download all by artist ID')
@click.option('-A', '--album-id', metavar='ID', type=int, help='Download by album ID')
@click.option('-u', '--url', metavar='URL', help='Download by URL of artist/album/track')
@click.option('-f', '--file', 'input_file', metavar='FILE', help='Download batch of artists from file, one per line')
@click.option('-b', '--bitrate', metavar='N', type=int, default=config["bitrate"],
              help='Set custom bitrate for this operation')
@click.option('-r', '--record-type', metavar='TYPE', default=config["record_type"],
              help='Only get certain record types')
def download_command(artist, artist_id, album_id, url, input_file, bitrate, record_type):
    """Download specific artist, album ID or by URL"""

    params = {
        'artist': artist,
        'artist_id': artist_id,
        'album_id': album_id,
        'url': url,
        'bitrate': bitrate,
        'record_type': record_type,
        'file': input_file
    }

    dl = download.Download()
    dl.download(params)


@run.command(name='monitor', context_settings={"ignore_unknown_options": True})
@click.argument('artist', nargs=-1)
@click.option('-i', '--artist-id', type=int, metavar="ID", help="Monitor artist by ID")
@click.option('-u', '--url', metavar="URL", help='Monitor artist by URL')
@click.option('-R', '--remove', is_flag=True, help='Stop montioring an artist')
def monitor_command(artist, artist_id, remove, url):
    """
    Monitor artist for new releases by ID, URL or name.

    \b
    Examples:
        monitor Mozart
        monitor --artist-id 100
        monitor --url https://www.deezer.com/us/artist/000
    """

    mon = monitor.Monitor()

    if url:
        id_from_url = url.split('/artist/')
        try:
            artist_id = int(id_from_url[1])
        except (IndexError, ValueError):
            logger.error(f"Invalid URL -- {url}")
            sys.exit(1)

    if artist_id:
        mon.artist_id = artist_id
    else:
        mon.artist = artist

    if remove:
        mon.stop_monitoring()
    else:
        mon.start_monitoring()

    refresh = Refresh()
    refresh.refresh()


@run.command(name='refresh')
def refresh_command():
    """Check artists for new releases"""
    refresh = Refresh()
    refresh.refresh()


@run.command(name='show')
@click.option('-a', '--artists', is_flag=True, help='Show artists currently being monitored')
@click.option('-n', '--new-releases', metavar='N', type=int, help='Show new releases from last N days')
@click.option('-s', '--stats', is_flag=True, help='Show various usage stats')
def show_command(artists, new_releases, stats):
    """
    Show monitored artists, latest new releases and various statistics
    """
    show = ShowStats()
    if artists:
        show.artists()
    elif new_releases:
        show.releases(new_releases)
    elif stats:
        show.stats()


@run.command(name='import')
@click.argument('path')
def import_cmd(path):
    """Import artists from CSV, text file or directory"""
    batch = BatchJobs()
    batch.import_artists(path)


@run.command()
@click.option('--include-logs', is_flag=True, help='include log files in backup')
def backup(include_logs):
    """Backup configuration and database to a tar file"""

    def filter_func(item):
        exclusions = ['deemon/backups']
        if not include_logs:
            exclusions.append('deemon/logs')
        if item.name not in exclusions:
            return item

    backup_tar = datetime.today().strftime('%Y%m%d-%H%M%S') + ".tar"
    backup_path = Path(settings.config_path / "backups")

    with tarfile.open(backup_path / backup_tar, "w") as tar:
        tar.add(settings.config_path, arcname='deemon', filter=filter_func)
        logger.info(f"Backed up to {backup_path / backup_tar}")




from pathlib import Path
from setuptools import setup, find_packages
from deemon import __version__
import os

with open('requirements.txt') as f:
    required = f.read().splitlines()

HERE = Path(__file__).parent
README = (HERE / "README.md").read_text()
DESCRIPTION = "Monitor new releases by a specified list of artists and auto download using the deemix library"

setup(
    name="deemon",
    version=__version__,
    author="digitalec",
    description=DESCRIPTION,
    long_description=README,
    long_description_content_type="text/markdown",
    license="GPL3",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.6",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=required,
    url="https://github.com/digitalec/deemon",
    entry_points = {
        "console_scripts": ["deemon=deemon.__main__:main"],
    }
)



from deemon import __version__
from deemon.app import utils
from datetime import datetime
from pathlib import Path
import logging
import sqlite3
import time


logger = logging.getLogger(__name__)


class DBHelper:

    def __init__(self, db=None):

        self.conn = None
        self.cursor = None
        if db:
            if not Path(db).exists():
                self.open(db)
                self.create_new_database()
            else:
                self.open(db)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def open(self, name):
        try:
            self.conn = sqlite3.connect(name)
            self.cursor = self.conn.cursor()
        except sqlite3.OperationalError as e:
            logger.error(f"Error opening database: {e}")

    def close(self):
        if self.conn:
            self.conn.commit()
            self.cursor.close()
            self.conn.close()

    def commit(self):
        self.conn.commit()

    def commit_and_close(self):
        logger.debug("Saving changes to DB")
        self.commit()
        self.close()

    def create_new_database(self):
        logger.debug("Updating database structure")

        sql_monitor = ("CREATE TABLE IF NOT EXISTS 'monitor' "
                       "('artist_id' INTEGER, 'artist_name' TEXT, 'bitrate' INTEGER, "
                       "'record_type' TEXT, 'alerts' INTEGER)")

        sql_releases = ("CREATE TABLE IF NOT EXISTS 'releases' "
                        "('artist_id' INTEGER, 'artist_name' TEXT, 'album_id' INTEGER, "
                        "'album_name' TEXT, 'album_release' TEXT, 'album_added' INTEGER, "
                        "'future_release' INTEGER DEFAULT 0)")

        self.query(sql_monitor)
        self.query(sql_releases)
        self.query("CREATE TABLE IF NOT EXISTS 'deemon' ('property' TEXT, 'value' TEXT)")
        self.query("CREATE UNIQUE INDEX 'idx_property' ON 'deemon' ('property')")
        self.query("CREATE UNIQUE INDEX 'idx_artist_id' ON 'monitor' ('artist_id')")
        self.query(f"INSERT INTO 'deemon' ('property', 'value') VALUES ('version', '{__version__}')")
        self.commit()

    def get_db_version(self):
        try:
            version = self.query(f"SELECT value FROM deemon WHERE property = 'version'").fetchone()[0]
        except sqlite3.OperationalError:
            version = '0.0.0'

        logger.debug(f"Database version {version}")
        return version

    def query(self, query: str, values=None):
        if values is None:
            values = {}
        result = self.cursor.execute(query, values)
        return result

    def reset_future(self, album_id):
        logger.debug("Clearing future_release flag from " + str(album_id))
        values = {'album_id': album_id}
        sql = "UPDATE 'releases' SET future_release = 0 WHERE album_id = :album_id"
        result = self.query(sql, values)

    def get_all_monitored_artists(self):
        '''
        Get unique set of artists stored in database

        :return: Unique set of all artists
        :rtype: set
        '''
        result = self.query(f"SELECT * FROM monitor")
        artists = set(x for x in result)
        return sorted(artists, key=lambda x: x[1])

    def get_specified_artist(self, artist):
        if type(artist) is int:
            values = {'artist': artist}
            result = self.query("SELECT * FROM monitor WHERE artist_id = :artist", values).fetchone()
        else:
            values = {'artist': artist}
            result = self.query("SELECT * FROM monitor WHERE artist_name = :artist COLLATE NOCASE", values).fetchone()
        return result

    def add_new_release(self, artist_id, artist_name, album_id, album_name, release_date, future_release):
        timestamp = int(time.time())
        values = {'artist_id': artist_id, 'artist_name': artist_name, 'album_id': album_id,
                  'album_name': album_name, 'release_date': release_date, 'future': future_release}
        sql = (f"INSERT INTO releases ('artist_id', 'artist_name', 'album_id', "
               f"'album_name', 'album_release', 'album_added', 'future_release') "
               f"VALUES (:artist_id, :artist_name, :album_id, :album_name, "
               f":release_date, {timestamp}, :future)")
        self.query(sql, values)

    def is_monitored(self, artist_id):
        result = self.query(f"SELECT artist_id, artist_name FROM monitor WHERE artist_id = {artist_id}").fetchone()
        if result:
            logger.info(f"Artist {result[1]} ({result[0]}) is already being monitored")
            return True

    def show_new_releases(self, from_date_ts, now_ts):
        today_date = datetime.utcfromtimestamp(now_ts).strftime('%Y-%m-%d')
        from_date = datetime.utcfromtimestamp(from_date_ts).strftime('%Y-%m-%d')
        values = {'from': from_date, 'today': today_date}
        sql = "SELECT * FROM 'releases' WHERE album_release >= :from AND album_release <= :today"
        result = self.query(sql, values)
        return result

    def get_artist_by_id(self, artist_id):
        values = {'id': artist_id}
        sql = "SELECT * FROM 'releases' WHERE artist_id = :id"
        result = self.query(sql, values).fetchone()
        return result

    def get_album_by_id(self, album_id):
        values = {'id': album_id}
        sql = "SELECT * FROM 'releases' WHERE album_id = :id"
        result = self.query(sql, values).fetchone()
        return result
